# Zope
from Globals import InitializeClass,PersistentMapping
from ZODB import PersistentList
from AccessControl import ClassSecurityInfo
from Products.PageTemplates.PageTemplateFile import PageTemplateFile
from Persistence import Persistent
from OFS.SimpleItem import SimpleItem

# Silva
from Products.Silva import SilvaPermissions
from Products.Silva import mangle
from Products.Silva.Asset import Asset
from Products.Silva.interfaces import IAsset
from Products.Silva.helpers import add_and_edit
from DateTime import DateTime

icon = "www/daffodil3.gif"

addable_priority = 1

manage_addStylesheetForm = PageTemplateFile("www/stylesheetAdd", globals(),
                                            __name__='manage_addStylesheetForm')

def manage_addStylesheet(self, id, title, REQUEST=None):
    """Add a stylesheet"""
    if not mangle.Id(self,id).isValid():
        return
    object = Stylesheet(id)
    self._setObject(id, object)
    object = getattr(self, id)
    object.set_title(title)
    add_and_edit(self, id, REQUEST)
    return ''

"""Stylesheet uses formulator forms to display the various css declarations in Silva.
Some of these css declarations contain characters that can't be part of name and id attributes.  Example:
div.citation p
a:hover
.margins
' ' and ':' are invalid characters.  To get around this, the id's of the formulator fields containing these are modified, such that:
' ' = ..
':' = __
^\. = zz.

When the stylesheet is rendered, the '..' and '__' are replaced with ' ' and ':'.  The process is reveresed when saving the stylesheets.

To customize this for your environment, there is one value in Stylesheet that needs to be updated.  In __init__, self._css_prefix needs to be updated.  This value is prefixed to every style in the stylesheet.  E.g.  The default '#bu-content-pad' is prefixed to every style, like:
#bu-content-pad strong { ... }
"""

class Stylesheet(Asset, SimpleItem):
    __doc__ = "A Stylesheet Asset that is editable in silva.  It allows you to customize the various aspects of Silva Document content"

    security = ClassSecurityInfo()

    meta_type = "Silva Stylesheet"

    __implements__ = IAsset

    def __init__(self, id):
        Stylesheet.inheritedAttribute('__init__')(self, id, '[Title is stored in metadata.  This is a bug.]')
        #may will be used for caching
        self._previous_lastmod = DateTime()
        self._cached_css = ''
        self._css = PersistentMapping()
        self._css_prefix = '#bu-content-pad'

    # ACCESSORS
    security.declareProtected(SilvaPermissions.View, 'is_cacheable')
    def is_cacheable(self):
        #could be cacheable, if the modified time is compared to a last modified time...
        return 1

    security.declareProtected(SilvaPermissions.AccessContentsInformation,
                              'is_deletable')
    def is_deletable(self):
        """always deletable"""
        return 1

    security.declareProtected(SilvaPermissions.AccessContentsInformation,
                              'can_set_title')
    def can_set_title(self):        
        """always settable"""
        # XXX: we badly need Publishable type objects to behave right.
        return 1


    #will raise an attribute error if a style contains invalid characters.
    #currently this is just '}'.
    security.declareProtected(SilvaPermissions.ChangeSilvaContent, 'save_styles')
    def save_styles(self, set, styles):
        from zLOG import LOG,INFO
        LOG('Stylesheet',INFO,'set is: ' + str(set))
        #set is the css set (e.g. paragraph, heading, anchor, etc)
        #styles is a formulator results hash.
        if not hasattr(self._css,set):
            self._css[set] = PersistentMapping()
        css_set = self._css[set]
        for (style,decls) in styles.items():
            for d in decls:
                if set!='custom' and d.find('}') > -1:
                    raise AttributeError('Invalid character "}" in style declaration.')
            css_set[style] = PersistentList.PersistentList(decls)

    security.declareProtected(SilvaPermissions.ChangeSilvaContent, 'save_css_prefix')
    def save_css_prefix(self, prefix):
        #css prefix is appended to every style decl, like 'css_prefix a:link {}'
        self._css_prefix = str(prefix).strip()

    security.declareProtected(SilvaPermissions.AccessContentsInformation, 'get_css_prefix')
    def get_css_prefix(self):
        return self._css_prefix

    #returns a list containing set['styles'].
    security.declareProtected(SilvaPermissions.AccessContentsInformation,
                              'get_style')
    def get_style(self, set, style):
        return '\n'.join(self._css.get(set,{}).get(style,[]))

    security.declareProtected(SilvaPermissions.View, 'index_html')
    def index_html(self, view_method=None, REQUEST=None):
        """View CSS.  The view_method param is ignored."""
        if REQUEST is None:
            REQUEST = self.REQUEST
        REQUEST.RESPONSE.setHeader('Content-Type','text/css; charset=utf8')
        #chech to see if content has been modified lately.
        if self.bobobase_modification_time() > self._previous_lastmod:
            self._previous_lastmod = self.bobobase_modification_time()
            self._cached_css = self.render_css()
        return self._cached_css

    security.declareProtected(SilvaPermissions.AccessContentsInformation,
                              'render_css')
    def render_css(self, linebreak='\n'):
        css = []
        for set in self._css.values():
            for (style,decls) in set.items():
                if decls:
                    css.append(self.render_one_style(style,decls,linebreak))
        return linebreak.join(css)

    security.declareProtected(SilvaPermissions.AccessContentsInformation,
                              'render_one_style')
    def render_one_style(self, style, decls, linebreak):
        name = self.unmangle_style_name(style)
        if self._css_prefix:
            name = '%s %s'%(self._css_prefix,name)
        if style=='custom':
            decls = ''.join(decls).split('}')
            decls = [ "%s %s}"%(self._css_prefix,d) for d in decls if d]
            return linebreak.join(decls)
        return '%s {\n %s\n}'%(name,linebreak.join(decls))

    security.declareProtected(SilvaPermissions.AccessContentsInformation,
                              'unmangle_style_name')
    def unmangle_style_name(self, style):
        return style.replace('__',':').replace('..',' ').replace('zz.','.')
            
InitializeClass(Stylesheet)
